import time

def NormalisedTime():
    A = time.localtime(time.time())
    return A[0], A[1], A[2], 0, 0, 0, A[6], A[7], A[8]

##################################################################
# emma.day(), emma.month(), emma.year() ##########################
##################################################################

Day = emma.day
emma.register_func("Day",\
   "Requires no arguments\n\n"\
   "Returns the day of month on which the transaction happened\n\n"\
   "Example : Day() <= 15  -  only first half of a month");

Month = emma.month
emma.register_func("Month",\
   "Requires no arguments\n\n"\
   "Returns the month (1-12) in which the transaction happened\n\n"\
   "Example : Month() == 12");

Year = emma.year
emma.register_func("Year",\
   "Requires no arguments\n\n"\
   "Returns the year in which the transaction happened\n\n"\
   "Example : Year() == 1998");


##################################################################
# CurDay(), CurMonth(), CurYear(), CurWeek() #####################
##################################################################

FirstDayOfToday = time.mktime(NormalisedTime())

def CurDay():
    return emma.date() >= FirstDayOfToday

emma.register_func("CurDay",\
   "Requires no arguments\n\n"\
   "Returns True if the transaction happened today\n\n"\
   "Example : CurDay()");

A = NormalisedTime()
FirstDayOfMonth = time.mktime(A[0], A[1], 1, A[3], A[4], A[5], A[6], A[7], A[8]);

def CurMonth():
    return emma.date() >= FirstDayOfMonth

emma.register_func("CurMonth",\
   "Requires no arguments\n\n"\
   "Returns True if the transaction happened in the current month\n\n"\
   "Example : CurMonth()");

A = NormalisedTime()
FirstDayOfYear = time.mktime(A[0], 1, 1, A[3], A[4], A[5], A[6], A[7], A[8]);

def CurYear():
    return emma.date() >= FirstDayOfYear

emma.register_func("CurYear",\
   "Requires no arguments\n\n"\
   "Returns True if the transaction happened in the current year\n\n"\
   "Example : CurYear()");

FirstDayOfWeek = time.mktime(NormalisedTime()) - time.localtime(time.time())[6] * 60 * 60 * 24

def CurWeek():
    return emma.date() >= FirstDayOfWeek

emma.register_func("CurWeek",\
   "Requires no arguments\n\n"\
   "Returns True if the transaction happened in the current week\n\n"\
   "Example : CurWeek()");

##################################################################
# After(date_str), Before(date_str) ##############################
##################################################################

def After(date_str):
    return emma.date() >= emma.date_val(date_str)

emma.register_func("After",\
   "Arg : date string (string)\n\n"\
   "Returns True if the transaction happened after the given date\n\n")

def Before(date_str):
    return emma.date() <= emma.date_val(date_str)

emma.register_func("Before",\
   "Arg : date string (string)\n\n"\
   "Returns True if the transaction happened before the given date\n\n")

##################################################################
# AccFrom(acc_name), AccFromSubOf(par_name), AccFromIn(acc_set) ##
# AccTo(acc_name),   AccToSubOf(par_name),   AccToIn(acc_set)   ##
##################################################################

def AccFrom(acc_name):
    return emma.acc_from() == acc_name

emma.register_func("AccFrom",\
   "Arg : name of an account (string)\n\n"\
   "Returns True if the transaction takes money from the given account\n\n"\
   "Example : AccFrom(\"Cash\")");

def AccFromSubOf(par_name):
    return par_name in emma.parents(emma.acc_from())

emma.register_func("AccFromSubOf",\
   "Arg : name of an account (string)\n\n"\
   "Returns True if the transaction takes money from subaccount of the given account\n\n"\
   "Example : AccFromSubOf(\"Incomes\")");

def AccFromIn(acc_set):
    return emma.acc_from() in acc_set

emma.register_func("AccFromIn",\
   "Arg : list of names of accounts (list of strings)\n\n"\
   "Returns True if the transaction takes money from any of accounts on the list\n\n"\
   "Example : AccFromIn([\"Cash\",\"Credit\"])");

def AccTo(acc_name):
    return emma.acc_to() == acc_name

emma.register_func("AccTo",\
   "Arg : name of an account (string)\n\n"\
   "Returns True if the transaction gives money to the given account\n\n"\
   "Example : AccTo(\"Cash\")");

def AccToSubOf(par_name):
    return par_name in emma.parents(emma.acc_to())

emma.register_func("AccToSubOf",\
   "Arg : name of an account (string)\n\n"\
   "Returns True if the transaction gives money to subaccount of the given account\n\n"\
   "Example : AccToSubOf(\"Expenses\")");

def AccToIn(acc_set):
    return emma.acc_to() in acc_set

emma.register_func("AccToIn",\
   "Arg : list of names of accounts (list of strings)\n\n"\
   "Returns True if the transaction gives money to any of accounts on the list\n\n"\
   "Example : AccToIn([\"Bills\",\"Car\", \"Food\"])");

##################################################################
# Amount() #######################################################
##################################################################

Amount = emma.amount

emma.register_func("Amount",\
   "Requires no arguments\n\n"\
   "Returns an amount the transaction deals with\n\n"\
   "Example : Amount() > 100");

##################################################################
# DescMatch(reg_str) #############################################
##################################################################

import re

def DescMatch(reg_str):
    return re.search(reg_str, emma.desc()) <> None

emma.register_func("DescMatch", \
    "Arg : perl regular expression (string)\n\n"\
    "Returns true only if given regular expression matches the transaction description\n\n"\
    "Example : DescMatch(\"[Bb]read\")")


##################################################################
# Functions using DateTime below #################################
##################################################################

try:
    from DateTime import *
except:
    print "In order to use the limits you need the mxdatetime package"
 

today = (now()+RelativeDateTime(hour=0,minute=0,second=0))

##################################################################
# Day(), Month(), Year(), Week() #####################
##################################################################

def Days(days=1,offset=0):
    # first find this and next day
    this = (today + RelativeDateTime(days=offset))
    next = (today + RelativeDateTime(days=offset + days))
    return emma.date() >= this.ticks() and emma.date() < next.ticks()

emma.register_func("Days",\
   "Takes two arguments:\n"\
   "  numDays (default 1)\n"\
   "  offset (default 0)\n"\
   "Returns True if the transaction happened\n"\
   "between today plus offset days and the\n"\
   "next numDays\n\n"\
   "Examples : \n"\
   "  Days(2)    - gets next 2 days\n"\
   "  Days(7,-7) - gets last 7 days\n"\
   "  Days(1,-7) - gets a week ago today\n");

# -1: Sunday
#  0: Monday
startOfWeekOffset = -1

def Weeks(numWeeks=1,offset=0):
    # first find the first day of this and next week
    A = RelativeDateTime(days=7*offset-startOfWeekOffset)
    B = RelativeDateTime(days=7*(offset+numWeeks)-startOfWeekOffset)
    this = (today + RelativeDateTime(weekday=(Monday,0)) + A)
    next = (today + RelativeDateTime(weekday=(Monday,0)) + B)
    print 'this:', this.tuple(), 'next:', next.tuple()
    return emma.date() >= this.ticks() and emma.date() < next.ticks()

emma.register_func("Weeks",\
   "Takes two arguments:\n"\
   "  numWeeks (default 1)\n"\
   "  offset (default 0)\n"\
   "Returns True if the transaction happened\n"\
   "in the next <numWeeks> weeks offset by \n"\
   "<offset> weeks, starting on Monday\n\n"\
   "Examples : \n"\
   "  Weeks(2)    - gets this and next weeks\n"\
   "  Weeks(1,1)  - gets next weeks\n"\
   "  Weeks(4,-4) - gets last 4 weeks");


def Months(months=1,offset=0):
    # Find first day of this and next month
    this = (today+RelativeDateTime(months=offset,day=1))
    next = (today+RelativeDateTime(months=offset+months,day=1))
    return emma.date() >= this.ticks() and emma.date() < next.ticks()

emma.register_func("Months",\
   "Takes two arguments:\n"\
   "  numMonths (default 1)\n"\
   "  offset (default 0)\n"\
   "Returns True if the transaction happened\n"\
   "in the next <numMonths> months offset by\n"\
   "<offset> weeks.\n\n"\
   "Examples : \n"\
   "  Months(1,-1)  - gets last months transactions\n"\
   "  Months(2,-1)  - gets last and this months\n"\
   "  Months()      - gets this months transactions");

def Years(years=1,offset=0):
    this = DateTime(today.year+offset)
    next = DateTime(today.year+offset+years)
    return emma.date() >= this.ticks() and emma.date() < next.ticks()

emma.register_func("Years",\
   "Takes two arguments:\n"\
   "  numYears (default 1)\n"\
   "  offset (default 0)\n"\
   "Returns True if the transaction happened in\n"\
   "the next <numYears> year offset by <offset>\n"\
   "years.\n\n"\
   "Examples : \n"\
   "  Year()     - this years transactions\n"\
   "  Year(1,-1) - last years transactions\n"\
   "  Year(2,-4) - gets 2 years starting 4 years ago");





